import numpy as np
import pickle
import time
import copy
import sys

# ROS Libraries
import rospy
from std_msgs.msg import Bool
from trajectory_msgs.msg import (
    JointTrajectoryPoint,
    JointTrajectory
)
from ur_msgs.srv import SetIO

## Our libraries
from utils import load_plan, JTraj2ROS, speed_multiplier, calibration_correction, calib_corr_single
from Controller import Controller
#import gcode2contour as gc


# NEED TO CHANGE
robot_num = 0 # 0 or 1
robot_desc_param = "/r{}/robot_description".format(robot_num+1)


def execute_plan(plan,
        controller,
        extrude=False,
        confirm=False,
        skip_first=1,
        move_speed=1,
        n_trajs=100,
        retry_time=0.1,
        arm=0, 
        max_traj_retries=20):
    """
    Executes a plan on MoveIt

    Single arm only
    skips the first n trajectories in the plan
    """
    # calibration correction of all trajectories
    print("Doing calibration correction...")
    for i,t in enumerate(plan.trajs[arm]):
        print("Calibrated trajectory {} of {}".format(i, len(plan.trajs[arm])))
        plan.trajs[arm][i] = calibration_correction(speed_multiplier(t, 1/move_speed), robot_desc_param)

    # plan to the start position of the plan
    controller.group.go(calib_corr_single(plan.trajs[arm][skip_first].positions[0],"/r1/robot_description"))

    # execute trajectories in plan
    sleep_for = 0
    for i, t in enumerate(plan.trajs[arm]):
        print("---")

        if i == n_trajs + skip_first:
            print("Finishing prematurely, completed {} trajectories".format(i - skip_first))
            break

        if i < skip_first:
            print("skipping traj {}".format(i))
            continue

        if confirm:
            a = raw_input('Press enter to continue')
            if a == 'q':
                break
        else:
            time.sleep(sleep_for)

        print("Printing {}th trajectory in the plan, which is contour {}".format(i, t.contour))

        # Extruding with trajectory
        retries = 0
        while not controller.exec_ctraj(t, wait=False) \
                and retries < max_traj_retries:
            retries += 1
            time.sleep(retry_time)

        sleep_for = t.time[-1] - retry_time

        if retries == max_traj_retries:
            print("\nFailed after {} retries, quitting".format(max_traj_retries))
            sys.exit(0)
    return


def gohome(arm):
    con.group.go(plan.trajs[arm][1].positions[0])


if __name__ == '__main__':
    plan = load_plan('cal_points')

    # Test calibration
    con = Controller(robot=robot_num+1, disable_extruder=True) # one arm when both arms running
    #con = Controller(disable_extruder=True)         # any when one arm  running

    execute_plan(plan, con, confirm=True, arm=robot_num, extrude=False)
