import numpy as np
import pickle
from . import JTrajectory
#import gcode2contour as gc


def _emptyTraj(joints, arm):
    """
    Creates an JTrajectory with one point
    """
    traj = JTrajectory([joints], [0], arm)
    return traj


class Plan:
    """
    Holds the execution plan for n arms
    """

    def __init__(self,
            start_joints
            ):

        self.n = len(start_joints)
        self.start_joints = start_joints

        self.trajs = {i:[_emptyTraj(self.start_joints[i],i)] for i in range(self.n)}
                # time at the end of each trajectory
        self.cumul_time = {i:[0] for i in range(self.n)}


    def jointsAtTime(self, t, arm):
        """
        state of joints for an arm given time from start
        """

        # if no plan yet, still at start_joints
        if len(self.trajs[arm])==1 or t==0:
            return self.start_joints[arm]

        nth_traj = bisect_left(self.cumul_time[arm], t)
        if nth_traj == len(self.cumul_time[arm]):
            # return the final state of arm
            return self.trajs[arm][-1].positions[-1]

        # else search for time in nth_traj
        # TODO can switch this to bisect too
        tp = self.trajs[arm][nth_traj].time
        c_t = self.cumul_time[arm][nth_traj-1]
        wp_i = next(i for i,v in enumerate(tp) if v + c_t >= t)

        # linear interpolate between the waypoints
        j1 = np.array(self.trajs[arm][nth_traj].positions[wp_i-1])
        j2 = np.array(self.trajs[arm][nth_traj].positions[wp_i])
        t1 = tp[wp_i-1] + c_t
        t2 = tp[wp_i] + c_t

        return (t-t1)/(t2-t1)*(j2-j1) + j1


    def getLastJoint(self, arm):
        """
        The joint position arm is at the end of the plan so far
        """
        return self.jointsAtTime(self.cumul_time[arm][-1], arm)


def load_plan(model, contours=False):
    """
    load a saved plan

    You cannot have existing pybullet instance going
    """
    plan_file = 'plans/' + model + ".plan"

    with open(plan_file, 'rb') as f:
        _, _, start_joints, ctime, trajs = pickle.load(f)

    plan = Plan(start_joints)
    plan.trajs = trajs
    plan.cumul_time = ctime

    if contours:
        # Read gcode file
        #contours = gc.decode_gcode("plans/" + model + ".gcode", max_wp_dist=20)
        return plan, contours
    else:
        return plan
